#!/usr/bin/env python2.4

##system-logviewer
##
## Copyright (C) 2001-2004 Red Hat, Inc.
## Copyright (C) 2001-2004 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Last Modified May 1, 2003

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import gtk
import gobject
import os

import LogViewerGui
import LogBuffer

##
## I18N
## 
import gettext
gettext.bindtextdomain ("system-logviewer", "/usr/share/locale")
gettext.textdomain ("system-logviewer")
_=gettext.gettext

blankPixbuf = None      
try:
    blankPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-logviewer/pixmaps/blank.png")
except:
    pass
alertPixbuf = None      
try:
    alertPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-logviewer/pixmaps/alert.png")
except:
    pass

warningPixbuf = None      
try:
    warningPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-logviewer/pixmaps/warning.png")
except:
    pass

class LogFileClass:

    def __init__(self, prefName):
        self.buffer = LogBuffer.LogBuffer()
        self.searchBuffer = LogBuffer.LogBuffer()
        self.messages = []
        self.file_stat = []
        self.prefName = prefName
        self.fileNotFound = 0
        self.read_log(self.prefName)

    def set_messages(self, lines):
        self.messages = lines

    def get_messages(self):
        return self.messages

    def print_messages(self):
        for line in self.messages:
            print line

    def set_file_stat(self, stat):
        self.file_stat = stat
    
    def get_file_stat(self):
        return self.file_stat()

    def get_file_stat_st_mtime(self):
        return self.file_stat[8]
    
    def read_log(self, prefName):
        try:
            #open log file
            if prefName =="TUXLOG":
                if  os.access("/usr/sbin/tux2w3c", os.F_OK) == 1:
                    #uncompress TUX log file
                    log = os.popen("/usr/sbin/tux2w3c " + LogViewerGui.logDict[prefName][0])
            else:
                log = open(LogViewerGui.logDict[prefName][0], "r")
            #end of opening log file

            #store each line as a message in the class instance
            self.set_messages(log.readlines())

            #close log file
            log.close()
            #get initial stat for first auto_refresh call
            self.set_file_stat(os.stat(LogViewerGui.logDict[prefName][0]))
        except:
            #log file not found
            self.buffer.set_buffer_text(_("File not found"))
            self.fileNotFound = 1

        #store contents into messages and buffer
        if self.get_messages():
            iter = self.buffer.get_iter_at_offset (0)
            for line in self.get_messages():
                #determine if alert word or warning word
                alert = 0
                warning = 0
                if len(LogViewerGui.alertWords) > 0:
                    for word in LogViewerGui.alertWords:
                        if (string.find(line.lower(), word.lower()) > -1):
                            self.buffer.insert_pixbuf(iter, alertPixbuf)
                            alert = 1
                            break
                if alert == 0 and len(LogViewerGui.warningWords) > 0:
                    for word in LogViewerGui.warningWords:
                        if (string.find(line.lower(), word.lower()) > -1):
                            self.buffer.insert_pixbuf(iter, warningPixbuf)
                            warning = 1
                            break
                if alert == 0 and warning == 0:
                    self.buffer.insert_pixbuf(iter, blankPixbuf)
                    
                #insert log file line into buffer
                self.buffer.insert_into_buffer_at_offset(iter, line)

    def get_file_not_found(self):
        return self.fileNotFound

## Get status of log file

    def fileChanged(self):
        # TRUE = file exists and has changed
        try:
            #has file location changed?
            data, iter = LogViewerGui.logFileView.get_selection().get_selected()
            #3 value in liststore if filechanged flag
            if LogViewerGui.logFilesListStore.get_value(iter, 3) == 1:
                return gtk.TRUE
            #compare st_mtime from stat to previous st_mtime from current stat
            status = (os.stat(LogViewerGui.logDict[self.prefName][0]))
            if (status[8] != self.get_file_stat_st_mtime()):
                return gtk.TRUE
            else:
                return gtk.FALSE
        except:
            self.buffer.set_buffer_text(LogViewerGui.logDict[self.prefName][0] + _(" file not found") )
            return gtk.FALSE

## Refresh displayed content of log file
                    
    def auto_refresh(self):
        #temp variables only used in this function
        new_messages = None
        newTextBuffer = LogBuffer.LogBuffer()
        line_count = 0

        #read contents of changed file
        try:
            if self.prefName == "TUXLOG":
                log = os.popen("/usr/sbin/tux2w3c " + LogViewerGui.logDict[self.prefName][0])
            else:
                log = open(LogViewerGui.logDict[self.prefName][0], "r")
            new_messages = log.readlines()
            log.close()
        except:
            self.buffer.set_buffer_text( _("File not found while refreshing contents.") )
            return

        #push contents of file into newTextBuffer
        #newTextBuffer already as alert icons
        iter = newTextBuffer.get_iter_at_offset (0)
        for line in new_messages:
            alert = 0
            warning = 0
            if len(LogViewerGui.alertWords[0]) > 0:
                for word in LogViewerGui.alertWords:
                    if (string.find(line.lower(), word.lower()) > -1):
                        newTextBuffer.insert_pixbuf(iter, alertPixbuf)
                        alert = 1
                        break
            if alert == 0 and len(LogViewerGui.warningWords[0]) > 0:
                for word in LogViewerGui.warningWords:
                    if (string.find(line.lower(), word.lower()) > -1):
                        newTextBuffer.insert_pixbuf(iter, warningPixbuf)
                        warning = 1
                        break
            if alert == 0 and warning == 0:
                newTextBuffer.insert_pixbuf(iter, blankPixbuf)

            newTextBuffer.insert_into_buffer_at_offset(iter, line)

        ##CASE 1: has file location changed? is so, replace displayed contents
        ##with new file and return
        data, iter = LogViewerGui.logFileView.get_selection().get_selected()
        if LogViewerGui.logFilesListStore.get_value(iter, 3) == 1:
            self.buffer = newTextBuffer
            replaced_buffer = 1
            #reset filechanged flag
            LogViewerGui.logFilesListStore.set_value(iter, 3, 0)
            #sync stored messages with contents of file
            self.set_messages(new_messages)
            #set previous stat to current stat for next refresh
            self.set_file_stat(os.stat(LogViewerGui.logDict[self.prefName][0]))
            #FIXME: use existing filter if it exists
            return

        ##CASE 2: reset buffer to new buffer
        self.buffer = newTextBuffer
        #sync stored messages with contents of file
        self.set_messages(new_messages)
        #set previous stat to current stat for next refresh
        self.set_file_stat(os.stat(LogViewerGui.logDict[self.prefName][0]))
        #FIXME: use existing filter if it exists
