#!/usr/bin/env python2.4

##system-logviewer
##
## Copyright (C) 2001-2004 Red Hat, Inc.
## Copyright (C) 2001-2004 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Created June 22, 2002
#Last Modified May 1, 2003

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import gtk
import gobject
import os

import LogViewerGui
import LogBuffer

##
## I18N
## 
import gettext
gettext.bindtextdomain ("system-logviewer", "/usr/share/locale")
gettext.textdomain ("system-logviewer")
_=gettext.gettext

blankPixbuf = None      
try:
    blankPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-logviewer/pixmaps/blank.png")
except:
    pass
alertPixbuf = None      
try:
    alertPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-logviewer/pixmaps/alert.png")
except:
    pass
warningPixbuf = None      
try:
    warningPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-logviewer/pixmaps/warning.png")
except:
    pass

class LogFileFrame:

    def __init__(self, frameName, description, logClassInstance):
        self.logClassInstance = logClassInstance

        #--------Create Frame -------------------------------------
        
        self.logVBox = gtk.VBox(gtk.FALSE, 6)
        self.logVBox.set_border_width(6)
        self.logFrame = gtk.Frame(frameName)
        self.logFrame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.logFrame.set_border_width(6)        

        self.logFrame.add(self.logVBox)

        self.descriptionHBox = gtk.HBox(gtk.FALSE, 6)
        self.descriptionHBox.set_border_width(2)

        icon = None      
        try:
            icon = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/system-logviewer.png")
        except:
            pass

        image = gtk.Image()
        if icon:
            image.set_from_pixbuf(icon)

        self.descriptionHBox.pack_start(image, gtk.FALSE, gtk.FALSE)
        label = gtk.Label(description)
#        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_line_wrap(gtk.TRUE)
        label.set_size_request(350, -1)
        self.descriptionHBox.pack_start(label, gtk.TRUE, gtk.TRUE, 6)

        self.logVBox.pack_start(self.descriptionHBox, gtk.FALSE)

        self.logScrollwindow = gtk.ScrolledWindow()
        self.logScrollwindow.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.logScrollwindow.set_shadow_type (SHADOW_ETCHED_IN)
        
        self.logView = gtk.TextView()

        #populate buffer with buffer from logClassInstance
        self.logView.set_buffer(logClassInstance.buffer.get_buffer())
           
        self.logView.set_editable(gtk.FALSE)
        self.logView.set_cursor_visible(gtk.FALSE)
        
        self.logScrollwindow.add(self.logView)
        self.logVBox.pack_start(self.logScrollwindow)        

        self.searchHBox = gtk.HBox(gtk.FALSE, 6)
        self.searchLabel = gtk.Label(_("Filter for: "))
        self.searchHBox.pack_start(self.searchLabel)
        self.searchEntry = gtk.Entry()
        self.searchEntry.connect ("activate", self.on_search_clicked)
        self.searchHBox.pack_start(self.searchEntry, gtk.TRUE, gtk.TRUE)
        self.searchButton = self.button_with_pixbuf("/usr/share/system-logviewer/pixmaps/search-icon.png", _("Filter"))
        self.searchButton.connect("clicked", self.on_search_clicked)
        self.searchHBox.pack_start(self.searchButton)        
        self.logVBox.pack_start(self.searchHBox, gtk.FALSE)        

        self.resetButton = self.button_with_stock(gtk.STOCK_REFRESH, _("Reset"))
        self.resetButton.connect("clicked", self.on_reset_clicked)
        self.searchHBox.pack_start(self.resetButton, gtk.TRUE, gtk.TRUE)        

    def get_frame(self):
        return self.logFrame

    def set_buffer(self, buffer):
        self.logView.set_buffer(buffer)

    def button_with_pixbuf (self, pixbuf, labeltext):
        button = gtk.Button ()
        label = gtk.Label (labeltext)
        hbox = gtk.HBox (gtk.FALSE, 2)
        align = gtk.Alignment (0.5, 0.5, 0.0, 0.0)
        icon = None      
        try:
            icon = gtk.gdk.pixbuf_new_from_file(pixbuf)
        except:
            pass

        image = gtk.Image()
        if icon:
            image.set_from_pixbuf(icon)        
            hbox.pack_start (image)
        
        hbox.pack_end (label)
        button.add (align)
        align.add (hbox)
        button.show_all ()
        return button

    def button_with_stock (self, stock_icon, labeltext):
        button = gtk.Button ()
        label = gtk.Label (labeltext)
        hbox = gtk.HBox (gtk.FALSE, 2)
        align = gtk.Alignment (0.5, 0.5, 0.0, 0.0)
        image = gtk.Image()
        if stock_icon:
            image.set_from_stock(stock_icon, 2)
            hbox.pack_start (image)
        
        hbox.pack_end (label)
        button.add (align)
        align.add (hbox)
        button.show_all ()
        return button

## Search button callback

    def on_search_clicked(self, *args):
        ###open messages list
        ###break into readlines
        ###search for entry text
        ###if search success: write to buffer + reset text view buffer
        ###if search failed: set status bar to Search Failed.

        LogViewerGui.set_status(_("Searching file..."))

        searchResultsBuffer = LogBuffer.LogBuffer()
        success = 0
        temp_messages = None
        if self.searchEntry.get_text().strip() != "":
            searchString = self.searchEntry.get_text().strip()
            temp_messages = self.logClassInstance.get_messages()
        else:
            LogViewerGui.set_status(_("Please enter a search string."))

        if temp_messages:
            iter = searchResultsBuffer.get_iter_at_offset (0)
            for line in temp_messages:
                alert = 0
                warning = 0
                #insert log file line into buffer if matches search text
                if string.find(line, searchString.strip()) > -1:
                    if len(LogViewerGui.alertWords[0]) > 0:
                        for word in LogViewerGui.alertWords:
                            if (string.find(line.lower(), word.lower()) > -1):
                                searchResultsBuffer.insert_pixbuf(iter, alertPixbuf)
                                alert = 1
                                break
                    if alert == 0 and  len(LogViewerGui.warningWords[0]) > 0:
                        for word in LogViewerGui.warningWords:
                            if (string.find(line.lower(), word.lower()) > -1):
                                searchResultsBuffer.insert_pixbuf(iter, warningPixbuf)
                                warning = 1
                                break
                    if alert == 0 and warning == 0:
                        searchResultsBuffer.insert_pixbuf(iter, blankPixbuf)

                    searchResultsBuffer.insert_into_buffer_at_offset(iter, line)
                    # set status that search was successful    
                    success = 1

            #if search is successful
            if success == 1:
                self.logClassInstance.searchBuffer = searchResultsBuffer
                self.logView.set_buffer(self.logClassInstance.searchBuffer.get_buffer())   

                LogViewerGui.set_status(_("Search results for ") + searchString + _(" displayed."))

            else:
                LogViewerGui.set_status(searchString + _(" not found."))

    def on_reset_clicked(self, *args):
        LogViewerGui.set_status("Reseting log file...")
        if self.logClassInstance.get_messages():
            self.logView.set_buffer(self.logClassInstance.buffer.get_buffer())
            self.logClassInstance.auto_refresh()

        LogViewerGui.set_status(_("Contents reset."))
