#!/usr/bin/env python2.4

##system-logviewer
##
## Copyright (C) 2001-2004 Red Hat, Inc.
## Copyright (C) 2001-2004 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Created December 12, 2001
#Last Modified January 23, 2003

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import sys
import gtk
import signal
import gobject
import Preferences
import os

import LogFileClass
import LogFileFrame
import LogBuffer

##
## I18N
## 
import gettext
gettext.bindtextdomain ("system-logviewer", "/usr/share/locale")
gettext.textdomain ("system-logviewer")
_=gettext.gettext

##---------------------------------------------------------------

##
## Icon for windows
##

iconPixbuf = None      
try:
    iconPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/system-logviewer.png")
except:
    pass

##---------------------------------------------------------------

##
## Lookup structure for left-side descriptions and frame titles
##

#stringDict is { KEYWORD: [name, description] }
stringDict = {}
stringDict["SYSLOG"] = [_("System Log"),_("This is the system log file.")]
stringDict["SECURELOG"] = [_("Security Log"),_("This log file contains security messages.")]
stringDict["MAILLOG"] = [_("Mail Log"),_("This log file records the mail server transactions.")]
stringDict["CRONLOG"] = [_("Cron Log"),_("This log file records when cron tasks are executed.")]
stringDict["NEWSLOG"] = [_("News Log"), _("This log file contains messages from the news server.")]
stringDict["BOOTLOG"] = [_("Boot Log"), _("This log file contains the boot messages.")]
stringDict["CIPELOG"] = [_("CIPE Log"),_("This log file displays messages for the CIPE interface.")]
stringDict["DMESGLOG"] = [_("Kernel Startup Log"),_("This log file contains the startup messages from the kernel (dmesg).")]
stringDict["HTTPDACCESSLOG"] = [_("Apache Access Log"),_("The Apache access log records the transfer of data from the server to the client.")]
stringDict["HTTPDERRORLOG"] = [_("Apache Error Log"),_("The Apache error log records errors during server and client transactions.")]
stringDict["MYSQLLOG"] = [_("MySQL Server Log"),_("This log file contains messages for the mySQL Server.")]
stringDict["RPMLOG"] = [_("RPM Packages"),_("This log file contains a list of RPM packages installed on the system. This list is updated daily by an automated cron task.")]
stringDict["TUXLOG"] = [_("Content Accelerator Log"),_("This file records messages for the Red Hat Content Accelerator Web Server.")]
stringDict["UP2DATELOG"] = [_("Update Agent Log"),_("The file contains a list of actions performed by the Red Hat Update Agent.")]
stringDict["FTPLOG"] = [_("FTP Log"),_("This log file contains messages from the FTP Server.")]
stringDict["XFREE86LOG"] = [_("XFree86 Log"),_("This log file contains messages from the XFree86 server.")]
stringDict["XORGLOG"] = [_("X.Org X11 Log"),_("This log file contains messages from the X.Org X11 server.")]

##---------------------------------------------------------------

#--------Notebook that displays log files and search box----
logFileNotebook = gtk.Notebook()
logFileNotebook.set_show_tabs(gtk.FALSE)

##---------------------------------------------------------------

##
## Used when reading in preferences
##

#logDict is { KEYWORD: [file location, LogFileClass instance, LogFileFrame instance, description string] }
logDict = {}
alertWords = list()
warningWords = []
prefsNum = 0
RefreshRate = 30000

##---------------------------------------------------------------

##
## Shared ListStore with descriptive string, file location, keyword, location changed (0=FALSE), notebook page number
##

logFilesListStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_INT)

##---------------------------------------------------------------

##
## left-hand side TreeView with descriptive strings as items
##

logFileView = gtk.TreeView(logFilesListStore)
#logFileView.set_rules_hint(gtk.TRUE)

##---------------------------------------------------------------

##
## status bar set by multiple classes
##

statusBar = gtk.Statusbar()

##
## Set Statusbar function
##

def set_status(text):
    statusBar.pop(0)
    statusBar.push(0, text)

##---------------------------------------------------------------
##---------------------------------------------------------------

##
## Main Class
##

class LogViewerGui:

## Destroy function for toplevel window

    def destroy(self, args):
        gtk.main_quit()

## Init

    def __init__ (self,width,height,allowConfigChanges, configFile,initialView):

        self.configFile = configFile

        self.initial_read_of_file()

        self.toplevel = gtk.Window()
        self.toplevel.connect ("destroy", self.destroy)
        self.toplevel.set_title(_("System Logs"))
        self.toplevel.set_default_size(width, height)
        self.toplevel.set_position(gtk.WIN_POS_CENTER)
        self.main_vbox = gtk.VBox(gtk.FALSE, 0)
        self.toplevel.add(self.main_vbox)
        self.toplevel.set_icon(iconPixbuf)

        #so the comment string for the desktop file will be translated
	commentString = _("Examine system log files")

        #--------Create Menubar------------------------------------
        accel_group = gtk.AccelGroup()
        item_fac = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)
        self.toplevel.add_accel_group(accel_group)
        if allowConfigChanges == 1:
            item_fac.create_items([
                ((_('/_File')),                 None,            None,                  0, '<Branch>'),
                ((_('/_File/_Refresh Now')),    '<control>R',    self.auto_refresh,     6, '<StockItem>', gtk.STOCK_REFRESH),
                ((_('/_File/_Quit')),            '<control>Q',    self.on_exit_selected, 6, '<StockItem>', gtk.STOCK_QUIT),
                ((_('/_Edit')),                 None,            None,                  0, '<Branch>'),
                ((_('/_Edit/_Preferences...')), None,            self.on_preferences_selected, 6, '<StockItem>', gtk.STOCK_PREFERENCES),
                
                ((_('/_Help')),                 None,            None,                  0, '<Branch>'),
                ((_('/_Help/_Contents')),  'F1',            self.on_manual_selected, 1, '<StockItem>', gtk.STOCK_HELP),
                ((_('/_Help/_About')),          None,            self.on_about_selected, 2, ''),
                ])
        else:
                        item_fac.create_items([
                ((_('/_File')),                 None,            None,                  0, '<Branch>'),
                ((_('/_File/_Refresh Now')),    '<control>R',    self.auto_refresh,     6, '<StockItem>', gtk.STOCK_REFRESH),
                ((_('/_File/_Quit')),            '<control>Q',    self.on_exit_selected, 6, '<StockItem>', gtk.STOCK_QUIT),
                ((_('/_Help')),                 None,            None,                  0, '<Branch>'),
                ((_('/_Help/_Contents')),  'F1',            self.on_manual_selected, 1, '<StockItem>', gtk.STOCK_HELP),
                ((_('/_Help/_About')),          None,            self.on_about_selected, 2, ''),
                ])

            
        self.menuBar = item_fac.get_widget('<main>')

        self.main_vbox.pack_start(self.menuBar, gtk.FALSE)

        #--------left list store of log file types------------------
        self.main_hpane = gtk.HPaned()
        self.main_vbox.pack_start(self.main_hpane)

        #--------status bar-----------------------------------------
        self.main_vbox.pack_start(statusBar, gtk.FALSE)

        #--------populate liststore with log file names----------------
        #--------add frame instances to notebook-----------------------        

        #populate list store with log location preferences
        keys_list = logDict.keys()
        keys_list.sort()
        #notebook pg number count
        self.pgNum = 0

        #set item 0 of logFilesListStore to descriptive string displayed on left side
        #add frame instance to notebook in same sort order so the notebook pg number is in sort order

        #set item 1 of logFilesListStore to file location
        #set item 2 of logFilesListStore to varname
        #set item 3 of logFilesListStore to location changed (0=FALSE)
        #set item 4 of logFilesListStore to notebook page number

        for key in keys_list:
            #if log file exists
            if os.access(logDict[key][0], os.F_OK) == 1:
                iter = logFilesListStore.append()
                logFileNotebook.append_page(logDict[key][2].get_frame(), gtk.Label(""))
                self.set_liststore_values(stringDict[key][0], key, iter)
      
        col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        logFileView.set_headers_visible(gtk.FALSE)
        logFileView.append_column(col)
        logFileView.set_border_width(8)


        logFileViewScrollWindow = gtk.ScrolledWindow()
        logFileViewScrollWindow.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        logFileViewScrollWindow.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        logFileViewScrollWindow.set_size_request(width/4,height)
        logFileViewScrollWindow.add(logFileView)
        
        self.main_hpane.add1(logFileViewScrollWindow)
        self.main_hpane.add2(logFileNotebook)
        selection = logFileView.get_selection()
        selection.connect("changed", self.select_category)
#!        iter = logFilesListStore.get_iter_root()

        #--------show gui---------------------------------------------
        self.toplevel.show_all()

#!        #--------select first log file in list------------------------
#!        selection.select_iter(iter)
      
        #--------select the initial view------------------------
        iter = logFilesListStore.get_iter_root()
        viewToSet = iter
         
        while (iter != None):
            if (initialView == logFilesListStore.get_value(iter, 0)):
                viewToSet = iter
                break
            iter = logFilesListStore.iter_next(iter)
 
        if (viewToSet != None):
 	    selection.select_iter(viewToSet)
            logFileView.set_cursor(logFilesListStore.get_path(viewToSet), None, gtk.FALSE)
  
        #--------keep track of last refresh rate pref-----------------
        self.lastRefreshRate = RefreshRate

        #--------continual updates------------------------------------
        gtk.timeout_add(RefreshRate, self.auto_refresh)

        #--------mainloop---------------------------------------------
        main ()


    def initial_read_of_file(self, *args):

        ##
        ## Read in preferences file
        ##

        try:
            prefsFile = open(self.configFile, "r")
            prefs = prefsFile.readlines()
            prefsFile.close()
        except:
            noPrefsDialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Preferences file not found."))
            noPrefsDialog.set_default_size(100,100)
            noPrefsDialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
            noPrefsDialog.set_border_width(2)
            noPrefsDialog.set_icon(iconPixbuf)
            rc = noPrefsDialog.run()
            noPrefsDialog.destroy()
            sys.exit(2)

        prefsNum = 0
        for line in prefs:
            line = string.strip(line)
            if line != "" and line[0]!="#":
                preference, values = string.split(line,"=")
                preference = string.strip(preference)
                if preference == "REFRESHRATE":
                    RefreshRateString = string.strip(values)
                    RefreshRate = int(RefreshRateString)*1000
        ##         elif preference == "SHOWDATE":
        ##             month,day,year,hour,minute = string.split(values,",")
        ##             month = string.strip(month)
        ##             day = string.strip(day)
        ##             year = string.strip(year)
        ##             hour = string.strip(hour)
        ##             minute = string.strip(minute)
                elif preference == "ALERTWORDS":
                    for word in string.split(values,","):
                        alertWords.append(word)
                elif preference == "WARNINGWORDS":
                    for word in string.split(values,","):
                        warningWords.append(word)
                #user-defined log files
                elif preference == "USERDEF":
                    #unique key for user-defined log file
                    preference = "USERDEF%s"%(prefsNum)
                    preference = string.strip(preference)
                    #add to stringDict because it is user-defined
                    stringDict[preference] = [string.split(values,",")[1],string.split(values,",")[2]]
                    #add location to logDict
                    logDict[preference] = [string.split(values,",")[0]]
                    #bump prefsNum
                    prefsNum = prefsNum + 1
                #the rest are default log file locations
                else:
                    #add location to logDict
                    logDict[preference] = [values]

        # Don't refresh faster than 30 seconds
        if RefreshRate < 30000:
            RefreshRate = 30000

        # Create LogFileClass instances after reading in alertWords so user-added alertWords are marked with icon
        # Create LogFileFrame instance per log file as well
        # Add them to logDict
        for key in logDict.keys():
            #only create class instance for files that exist
            if os.access(logDict[key][0], os.F_OK) == 1:
                logClassInstance = LogFileClass.LogFileClass(key)
                logFrame = LogFileFrame.LogFileFrame(stringDict[key][0], stringDict[key][1], logClassInstance)
                logDict[key] = logDict[key] + [logClassInstance, logFrame, stringDict[key][0]]

    
    def set_liststore_values(self, string, key, iter):
        logFilesListStore.set_value(iter, 0, string)
        logFilesListStore.set_value(iter, 1, str(logDict[key][0]))
        logFilesListStore.set_value(iter, 2, key)
        logFilesListStore.set_value(iter, 3, 0)
        logFilesListStore.set_value(iter, 4, self.pgNum)
        self.pgNum = self.pgNum + 1

## About Menu Item callback

    def on_about_selected(self, action, widget):
        dlg = gtk.MessageDialog (None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                                 _("Log Viewer 0.9.12\n Copyright (c) 2002-2004 Red Hat, Inc.\n Copyright (c) 2002-2004 Tammy Fox <tfox@redhat.com>\n A graphical interface for watching log files"))
        dlg.set_title(_("About Log Viewer"))
        dlg.set_default_size(100, 100)
        dlg.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        dlg.set_border_width(2)
        dlg.set_modal(gtk.TRUE)
        dlg.set_transient_for(self.toplevel)
        dlg.set_icon(iconPixbuf)
        rc = dlg.run()
        dlg.destroy()

## Manual Menu Item callback

    def on_manual_selected(self, action, widget):
        help_page = "file:///usr/share/doc/system-logviewer-" + "0.9.12" + "/index.html"
        path = "/usr/bin/htmlview"

        if path == None:
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("Help is not available.")))
            dlg.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
            dlg.run()
            dlg.destroy()
            return
		
        pid = os.fork()
        if not pid:
            os.execv(path, [path, help_page])

## Select Item from List Store callback

    def select_category(self, object):
        listStore, listStoreIter = object.get_selected()
        # set notebook page to selected log file
        try:
            logFileNotebook.set_current_page(logFilesListStore.get_value(listStoreIter, 4))
        except:
            #notebook might not contain the page anymore if it is deleted via preferences
            pass
        return

## Preferences Menu Item callback

    def on_preferences_selected(self, action, widget):
        prefsClass = Preferences.Preferences(self, self.configFile)
        prefsDialog = prefsClass.launch()
        prefsDialog.set_transient_for(self.toplevel)
        rc = prefsDialog.run()

        if rc == 0:
            prefsClass.on_close_clicked()
            set_status(_("Preferences updated."))
        elif rc == 1:
            prefsClass.on_cancel_clicked()
        
## Exit Menu Item callback

    def on_exit_selected(self, action, widget):
        gtk.main_quit()

## Auto reset function
        
    def auto_refresh(self, *args):
        #get liststore iter to refresh correct page
        listStore, listStoreIter = logFileView.get_selection().get_selected()
        # get current page number
        pageNum = logFilesListStore.get_value(listStoreIter, 4)
        # get unique key word such as SYSLOG
        logKeyWord = logFilesListStore.get_value(listStoreIter, 2)

        #restart timeout function if refresh rate pref has changed
        if RefreshRate != self.lastRefreshRate:
            gtk.timeout_add(self.lastRefreshRate, self.auto_refresh)
            self.lastRefreshRate = RefreshRate            
            return gtk.FALSE

        set_status(_("Refreshing log file..."))

        #call refresh if file changed
        if logDict[logKeyWord][1].fileChanged():
            logDict[logKeyWord][1].auto_refresh()
            #refresh frame buffer
            logDict[logKeyWord][2].set_buffer(logDict[logKeyWord][1].buffer.get_buffer())

        #reset status bar after auto_refresh
        set_status("")

        #keep track if refreshrate changes
        self.lastRefreshRate = RefreshRate
        #return TRUE so gtk.time_out will keep going
        return gtk.TRUE
    
